'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _possibleConstructorReturn2 = require('babel-runtime/helpers/possibleConstructorReturn');

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require('babel-runtime/helpers/inherits');

var _inherits3 = _interopRequireDefault(_inherits2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Filter = require('./Filter');

var _Filter2 = _interopRequireDefault(_Filter);

var _Matcher = require('./Matcher');

var _Matcher2 = _interopRequireDefault(_Matcher);

var _Parser = require('./Parser');

var _Parser2 = _interopRequireDefault(_Parser);

var _Element = require('./components/Element');

var _Element2 = _interopRequireDefault(_Element);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*:: import type {
  FilterStructure,
  FilterList,
  MatcherStructure,
  MatcherList,
  ParsedNodes,
} from './types';*/ /**
                     * @copyright   2016, Miles Johnson
                     * @license     https://opensource.org/licenses/MIT
                     * 
                     */

/* eslint-disable react/no-unused-prop-types */

var DEFAULT_PRIORITY /*: number*/ = 100;
var globalFilters /*: FilterList*/ = [];
var globalMatchers /*: MatcherList*/ = [];

function prioritySort(a /*: MatcherStructure | FilterStructure*/, b /*: MatcherStructure | FilterStructure*/) /*: number*/ {
  return a.priority - b.priority;
}

/*:: type InterweaveProps = {
  content: string,
  filters: Filter[],
  matchers: Matcher<*>[],
  noHtml: boolean,
  disableFilters: boolean,
  disableMatchers: boolean,
  onBeforeParse: (content: string) => string,
  onAfterParse: (content: ParsedNodes) => ParsedNodes,
  tagName: string,
};*/

var Interweave = function (_React$Component) {
  (0, _inherits3.default)(Interweave, _React$Component);

  function Interweave() {
    (0, _classCallCheck3.default)(this, Interweave);
    return (0, _possibleConstructorReturn3.default)(this, (Interweave.__proto__ || (0, _getPrototypeOf2.default)(Interweave)).apply(this, arguments));
  }

  (0, _createClass3.default)(Interweave, [{
    key: 'parseMarkup',


    /**
     * Parse the markup and apply hooks.
     */
    value: function parseMarkup() {
      var _props = this.props;
      var tagName = _props.tagName;
      var content = _props.content;
      var onBeforeParse = _props.onBeforeParse;
      var onAfterParse = _props.onAfterParse;
      var matchers = _props.matchers;
      var disableMatchers = _props.disableMatchers;
      var filters = _props.filters;
      var disableFilters = _props.disableFilters;
      var props = (0, _objectWithoutProperties3.default)(_props, ['tagName', 'content', 'onBeforeParse', 'onAfterParse', 'matchers', 'disableMatchers', 'filters', 'disableFilters']);


      var markup = content;

      if (onBeforeParse) {
        markup = onBeforeParse(markup);

        if (typeof markup !== 'string') {
          throw new TypeError('Interweave `onBeforeParse` must return a valid HTML string.');
        }
      }

      var newMatchers = disableMatchers ? [] : [].concat((0, _toConsumableArray3.default)(Interweave.getMatchers().map(function (row) {
        return row.matcher;
      })), (0, _toConsumableArray3.default)(matchers));

      var newFilters = disableFilters ? [] : [].concat((0, _toConsumableArray3.default)(Interweave.getFilters().map(function (row) {
        return row.filter;
      })), (0, _toConsumableArray3.default)(filters));

      markup = new _Parser2.default(markup, props, newMatchers, newFilters).parse();

      if (onAfterParse) {
        markup = onAfterParse(markup);

        if (!Array.isArray(markup)) {
          throw new TypeError('Interweave `onAfterParse` must return an array of strings and React elements.');
        }
      }

      return markup;
    }

    /**
     * Render the component by parsing the markup.
     *
     * @returns {JSX}
     */

  }, {
    key: 'render',
    value: function render() {
      return _react2.default.createElement(
        _Element2.default,
        { tagName: this.props.tagName },
        this.parseMarkup()
      );
    }
  }], [{
    key: 'addFilter',


    /**
     * Add a filter class that will be used to cleanse HTML attributes.
     *
     * @param {Filter} filter
     * @param {Number} [priority]
     */
    value: function addFilter(filter /*: Filter*/) {
      var priority /*: number*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      if (!(filter instanceof _Filter2.default)) {
        throw new TypeError('Filter must be an instance of the `Filter` class.');
      }

      // Apply and sort filters
      globalFilters.push({
        filter: filter,
        priority: priority || DEFAULT_PRIORITY + globalFilters.length
      });

      globalFilters.sort(prioritySort);
    }

    /**
     * Add a matcher class that will be used to match and replace tokens with components.
     *
     * @param {Matcher} matcher
     * @param {Number} [priority]
     */

    // eslint-disable-next-line react/sort-comp

  }, {
    key: 'addMatcher',
    value: function addMatcher(matcher /*: Matcher<*>*/) {
      var priority /*: number*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      if (!(matcher instanceof _Matcher2.default)) {
        throw new TypeError('Matcher must be an instance of the `Matcher` class.');
      }

      // Add a prop type so we can disable per instance
      Interweave.propTypes[matcher.inverseName] = _react.PropTypes.bool;

      // Append and sort matchers
      globalMatchers.push({
        matcher: matcher,
        priority: priority || DEFAULT_PRIORITY + globalMatchers.length
      });

      globalMatchers.sort(prioritySort);
    }

    /**
     * Reset all global filters.
     */

  }, {
    key: 'clearFilters',
    value: function clearFilters() {
      globalFilters = [];
    }

    /**
     * Reset all global matchers.
     */

  }, {
    key: 'clearMatchers',
    value: function clearMatchers() {
      globalMatchers = [];
    }

    /**
     * Configure default value for child props by
     * defining default props on the parent.
     *
     * @param {Object} props
     */

  }, {
    key: 'configure',
    value: function configure(props /*: Object*/) {
      Interweave.defaultProps = (0, _extends3.default)({}, Interweave.defaultProps, props);
    }

    /**
     * Return all defined filters for an attribute.
     *
     * @returns {{ filter: Filter }[]}
     */

  }, {
    key: 'getFilters',
    value: function getFilters() {
      return globalFilters;
    }

    /**
     * Return all defined matchers.
     *
     * @returns {{ matcher: Matcher }[]}
     */

  }, {
    key: 'getMatchers',
    value: function getMatchers() {
      return globalMatchers;
    }
  }]);
  return Interweave;
}(_react2.default.Component);

Interweave.propTypes = {
  content: _react.PropTypes.string.isRequired,
  filters: _react.PropTypes.arrayOf(_react.PropTypes.instanceOf(_Filter2.default)),
  matchers: _react.PropTypes.arrayOf(_react.PropTypes.instanceOf(_Matcher2.default)),
  noHtml: _react.PropTypes.bool,
  disableFilters: _react.PropTypes.bool,
  disableMatchers: _react.PropTypes.bool,
  onBeforeParse: _react.PropTypes.func,
  onAfterParse: _react.PropTypes.func,
  tagName: _react.PropTypes.oneOf(['span', 'div', 'p'])
};
Interweave.defaultProps = {
  filters: [],
  matchers: [],
  tagName: 'span'
};
exports.default = Interweave;