'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _Matcher = require('./Matcher');

var _Matcher2 = _interopRequireDefault(_Matcher);

var _Filter = require('./Filter');

var _Filter2 = _interopRequireDefault(_Filter);

var _Element = require('./components/Element');

var _Element2 = _interopRequireDefault(_Element);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*:: import type {
  Attributes,
  PrimitiveType,
  ParsedNodes,
  NodeConfig,
  NodeInterface,
} from './types';*/ /**
                     * @copyright   2016, Miles Johnson
                     * @license     https://opensource.org/licenses/MIT
                     * 
                     */

/* eslint-disable no-cond-assign, no-undef */

var ELEMENT_NODE /*: number*/ = 1;
var TEXT_NODE /*: number*/ = 3;

var Parser = function () {
  function Parser(markup /*: string*/) {
    var props /*: Object*/ = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var matchers /*: Matcher<*>[]*/ = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : [];
    var filters /*: Filter[]*/ = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : [];
    (0, _classCallCheck3.default)(this, Parser);

    this.doc = this.createDocument(markup);
    this.content = [];
    this.props = props;
    this.matchers = matchers;
    this.filters = filters;
    this.keyIndex = -1;
  }

  /**
   * Loop through and apply all registered attribute filters to the
   * provided value.
   *
   * @param {String} attribute
   * @param {String} value
   * @returns {String}
   */


  (0, _createClass3.default)(Parser, [{
    key: 'applyFilters',
    value: function applyFilters(attribute /*: string*/, value /*: string*/) {
      return this.filters.reduce(function (newValue, filter) {
        return filter.attribute === attribute ? filter.filter(newValue) : newValue;
      }, value);
    }

    /**
     * Loop through and apply all registered matchers to the string.
     * If a match is found, create a React element, and build a new array.
     * This array allows React to interpolate and render accordingly.
     *
     * @param {String} string
     * @param {Object} parentConfig
     * @returns {String|String[]}
     */

  }, {
    key: 'applyMatchers',
    value: function applyMatchers(string /*: string*/, parentConfig /*: NodeConfig*/) {
      var _this = this;

      var elements = [];
      var props = this.props;
      var matchedString = string;
      var parts = {};

      this.matchers.forEach(function (matcher) {
        var tagName = matcher.getTagName().toLowerCase();

        // Skip matchers that have been disabled from props or are not supported
        if (props[matcher.inverseName] || !_constants.TAGS[tagName]) {
          return;
        }

        var config = (0, _extends3.default)({}, _constants.TAGS[tagName], {
          tagName: tagName
        });

        // Skip matchers in which the child cannot be rendered
        if (config.rule === _constants.PARSER_DENY || !_this.canRenderChild(parentConfig, config)) {
          return;
        }

        // Continuously trigger the matcher until no matches are found
        while (parts = matcher.match(matchedString)) {
          var _parts = parts;
          var match = _parts.match;
          var partProps = (0, _objectWithoutProperties3.default)(_parts, ['match']);

          // Replace the matched portion with a placeholder

          matchedString = matchedString.replace(match, '#{{' + elements.length + '}}#');

          // Create an element through the matchers factory
          _this.keyIndex += 1;

          elements.push(matcher.createElement(match, (0, _extends3.default)({}, props, partProps || {}, {
            key: _this.keyIndex
          })));
        }
      });

      if (!elements.length) {
        return matchedString;
      }

      // Deconstruct the string into an array so that React can render it
      var matchedArray = [];
      var lastIndex = 0;

      while (parts = matchedString.match(/#\{\{(\d+)\}\}#/)) {
        var no = parts[1];
        // $FlowIssue https://github.com/facebook/flow/issues/2450
        var index = parts.index;

        // Extract the previous string
        if (lastIndex !== index) {
          matchedArray.push(matchedString.substring(lastIndex, index));
        }

        // Inject the element
        matchedArray.push(elements[parseInt(no, 10)]);

        // Set the next index
        lastIndex = index + parts[0].length;

        // Replace the token so it won't be matched again
        // And so that the string length doesn't change
        matchedString = matchedString.replace('#{{' + no + '}}#', '%{{' + no + '}}%');
      }

      // Extra the remaining string
      if (lastIndex < matchedString.length) {
        matchedArray.push(matchedString.substring(lastIndex));
      }

      return matchedArray;
    }

    /**
     * Determine whether the child can be rendered within the parent.
     *
     * @param {Object} parentConfig
     * @param {Object} childConfig
     * @returns {Boolean}
     */

  }, {
    key: 'canRenderChild',
    value: function canRenderChild(parentConfig /*: NodeConfig*/, childConfig /*: NodeConfig*/) {
      if (!parentConfig.tagName || !childConfig.tagName) {
        return false;
      }

      // Pass through first
      if (childConfig.rule === _constants.PARSER_PASS_THROUGH) {
        return false;
      }

      // Valid children second
      if (parentConfig.children.length && parentConfig.children.indexOf(childConfig.tagName) === -1) {
        return false;
      }

      // Self nesting third
      if (!parentConfig.self && parentConfig.tagName === childConfig.tagName) {
        return false;
      }

      // Block fourth
      if (!parentConfig.block && childConfig.type === _constants.TYPE_BLOCK) {
        return false;
      }

      // Inline last
      if (!parentConfig.inline && childConfig.type === _constants.TYPE_INLINE) {
        return false;
      }

      return true;
    }

    /**
     * Create a detached HTML document that allows for easy HTML
     * parsing while not triggering scripts or loading external
     * resources.
     *
     * @param {String} markup
     * @returns {HTMLDocument}
     */

  }, {
    key: 'createDocument',
    value: function createDocument(markup /*: string*/) {
      var doc = document.implementation.createHTMLDocument('Interweave');

      if (markup.substr(0, 9).toUpperCase() === '<!DOCTYPE') {
        doc.documentElement.innerHTML = markup;
      } else {
        doc.body.innerHTML = markup;
      }

      return doc;
    }

    /**
     * Convert an elements attribute map to an object map.
     *
     * @param {Node} node
     * @returns {Object}
     */

  }, {
    key: 'extractAttributes',
    value: function extractAttributes(node /*: NodeInterface*/) {
      var _this2 = this;

      var attributes = {};

      if (node.nodeType !== ELEMENT_NODE || !node.attributes) {
        return attributes;
      }

      (0, _from2.default)(node.attributes).forEach(function (attr) {
        if (!attr) {
          return;
        }

        var name /*: string*/ = attr.name.toLowerCase();
        var value /*: string*/ = attr.value;
        var filter /*: number*/ = _constants.ATTRIBUTES[name];

        // Do not allow blacklisted attributes excluding ARIA attributes
        // Do not allow events or XSS injections
        if (name.substr(0, 5) !== 'aria-') {
          if (!filter || filter === _constants.FILTER_DENY || name.match(/^on/) || value.match(/(javascript|script|xss):/i)) {
            return;
          }
        }

        // Apply filters
        var newValue /*: PrimitiveType*/ = _this2.applyFilters(name, value);

        // Cast to boolean
        if (filter === _constants.FILTER_CAST_BOOL) {
          newValue = newValue === 'true' || newValue === name;

          // Cast to number
        } else if (filter === _constants.FILTER_CAST_NUMBER) {
          newValue = parseFloat(newValue);

          // Cast to string
        } else {
          newValue = String(newValue);
        }

        attributes[_constants.ATTRIBUTES_TO_PROPS[name] || name] = newValue;
      });

      return attributes;
    }

    /**
     * Parse the markup by injecting it into a detached document,
     * while looping over all child nodes and generating an
     * array to interpolate into JSX.
     *
     * @returns {String[]|ReactElement[]}
     */

  }, {
    key: 'parse',
    value: function parse() {
      if (!this.content.length) {
        this.content = this.parseNode(this.doc.body, (0, _extends3.default)({}, _constants.TAGS.body, {
          tagName: 'body'
        }));
      }

      return this.content;
    }

    /**
     * Loop over the nodes children and generate a
     * list of text nodes and React elements.
     *
     * @param {Node} parentNode
     * @param {Object} parentConfig
     * @returns {String[]|ReactElement[]}
     */

  }, {
    key: 'parseNode',
    value: function parseNode(parentNode /*: NodeInterface*/, parentConfig /*: NodeConfig*/) {
      var _this3 = this;

      var noHtml = this.props.noHtml;

      var content = [];
      var mergedText = '';

      (0, _from2.default)(parentNode.childNodes).forEach(function (node) {
        // Create React elements from HTML elements
        if (node.nodeType === ELEMENT_NODE) {
          var tagName = node.nodeName.toLowerCase();

          if (!_constants.TAGS[tagName]) {
            return;
          }

          var config = (0, _extends3.default)({}, _constants.TAGS[tagName], {
            tagName: tagName
          });

          // Persist any previous text
          if (mergedText) {
            content.push(mergedText);
            mergedText = '';
          }

          // Skip over elements not supported
          if (config.rule === _constants.PARSER_DENY) {
            return;

            // Only pass through the text content
          } else if (noHtml || !_this3.canRenderChild(parentConfig, config)) {
            content = content.concat(_this3.parseNode(node, config));

            // Convert the element
          } else {
            _this3.keyIndex += 1;

            content.push(_react2.default.createElement(
              _Element2.default,
              {
                key: _this3.keyIndex,
                tagName: tagName,
                attributes: _this3.extractAttributes(node)
              },
              _this3.parseNode(node, config)
            ));
          }

          // Apply matchers if a text node
        } else if (node.nodeType === TEXT_NODE) {
          var text = _this3.applyMatchers(node.textContent, parentConfig);

          if (Array.isArray(text)) {
            content = content.concat(text);
          } else {
            mergedText += text;
          }
        }
      });

      if (mergedText) {
        content.push(mergedText);
      }

      return content;
    }
  }]);
  return Parser;
}();

exports.default = Parser;