'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EMOJI_PATTERN = exports.EMOJI_SHORTNAME_PATTERN = exports.SHORTNAME_TO_UNICODE = exports.UNICODE_TO_SHORTNAME = undefined;

var _fromCodePoint = require('babel-runtime/core-js/string/from-code-point');

var _fromCodePoint2 = _interopRequireDefault(_fromCodePoint);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _data = require('./data.json');

var _data2 = _interopRequireDefault(_data);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*:: type StringMap = { [key: string]: string };*/ /**
                                                    * @copyright   2016, Miles Johnson
                                                    * @license     https://opensource.org/licenses/MIT
                                                    * 
                                                    */

/*:: type EmojiMap = {
  [shortName: string]: {
    hexCode: string,
    codePoint: number[],
    unicode: string,
  }
};*/


var data /*: StringMap*/ = typeof _data2.default === 'string' ? JSON.parse(_data2.default) : _data2.default;
var emoji /*: EmojiMap*/ = {};

var UNICODE_TO_SHORTNAME /*: StringMap*/ = exports.UNICODE_TO_SHORTNAME = {};
var SHORTNAME_TO_UNICODE /*: StringMap*/ = exports.SHORTNAME_TO_UNICODE = {};

// Extract the shortname, codepoint, and unicode
// https://r12a.github.io/apps/conversion/
(0, _keys2.default)(data).forEach(function (name) {
  var hexCode = data[name];
  var shortName = ':' + name + ':';
  var codePoint = hexCode.split('-').map(function (point) {
    return parseInt(point, 16);
  });
  var unicode = _fromCodePoint2.default.apply(String, (0, _toConsumableArray3.default)(codePoint));

  UNICODE_TO_SHORTNAME[unicode] = shortName;
  SHORTNAME_TO_UNICODE[shortName] = unicode;

  emoji[shortName] = {
    hexCode: hexCode,
    codePoint: codePoint,
    unicode: unicode
  };
});

// These constants are located here so that they can be omitted from the bundle
var EMOJI_SHORTNAME_PATTERN /*: string*/ = exports.EMOJI_SHORTNAME_PATTERN = ':[-a-z0-9_]+:';
var EMOJI_PATTERN /*: string*/ = exports.EMOJI_PATTERN = '(?:0\u20E3|1\u20E3|2\u20E3|3\u20E3|4\u20E3|5\u20E3|6\u20E3|7\u20E3|8\u20E3|9\u20E3|\uD83D(?:\uDC69(?:\u2764\uD83D(?:\uDC69|\uDC8B\uD83D\uDC69)|\uD83D\uDC69\uD83D(?:\uDC66(?:\uD83D\uDC66)?|\uDC67(?:\uD83D(?:\uDC66|\uDC67))?)|\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF))|\uDC68(?:\u2764\uD83D(?:\uDC68|\uDC8B\uD83D\uDC68)|\uD83D(?:\uDC69\uD83D(?:\uDC67(?:\uD83D(?:\uDC66|\uDC67))?|\uDC66\uD83D\uDC66)|\uDC68\uD83D(?:\uDC66(?:\uD83D\uDC66)?|\uDC67(?:\uD83D(?:\uDC66|\uDC67))?))|\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF))|\uDC41\uD83D\uDDE8|\uDE4C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4F\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4B\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4E\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4A\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC4C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC50\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDCAA\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE4F\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC46\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC47\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC48\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC49\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD95\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD90\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD96\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC85\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC42\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC43\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC76\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC66\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC67\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC71\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC74\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC75\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC72\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC73\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC6E\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC77\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC82\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC7C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC78\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC70\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDEB6\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC83\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE47\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC81\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE45\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE46\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE4B\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE4E\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDE4D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC87\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDC86\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDEA3\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDEC0\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDEB4\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDEB5\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD75\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD7A\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF))|#\u20E3|\\*\u20E3|\uD83C(?:\uDDE6\uD83C(?:\uDDE8|\uDDEB|\uDDF1|\uDDE9|\uDDF4|\uDDEE|\uDDEC|\uDDF7|\uDDF2|\uDDFC|\uDDFA|\uDDF9|\uDDFF|\uDDEA|\uDDFD|\uDDF8|\uDDF6)|\uDDE9\uD83C(?:\uDDFF|\uDDF0|\uDDEF|\uDDF2|\uDDF4|\uDDEA|\uDDEC)|\uDDE7\uD83C(?:\uDDF8|\uDDED|\uDDE9|\uDDE7|\uDDFE|\uDDEA|\uDDFF|\uDDEF|\uDDF2|\uDDF9|\uDDF4|\uDDE6|\uDDFC|\uDDF7|\uDDF3|\uDDEC|\uDDEB|\uDDEE|\uDDF6|\uDDF1|\uDDFB)|\uDDE8\uD83C(?:\uDDFB|\uDDF2|\uDDE6|\uDDEB|\uDDF1|\uDDF3|\uDDF4|\uDDEC|\uDDE9|\uDDF7|\uDDFA|\uDDFE|\uDDFF|\uDDEE|\uDDED|\uDDFD|\uDDE8|\uDDF5|\uDDF0|\uDDFC)|\uDDF0\uD83C(?:\uDDED|\uDDFE|\uDDF2|\uDDFF|\uDDEA|\uDDEE|\uDDFC|\uDDEC|\uDDF5|\uDDF3|\uDDF7)|\uDDF9\uD83C(?:\uDDE9|\uDDFC|\uDDEF|\uDDFF|\uDDED|\uDDF1|\uDDEC|\uDDF4|\uDDF9|\uDDF3|\uDDF7|\uDDF2|\uDDFB|\uDDE6|\uDDF0|\uDDEB|\uDDE8)|\uDDED\uD83C(?:\uDDF7|\uDDF9|\uDDF3|\uDDF0|\uDDFA|\uDDF2)|\uDDEA\uD83C(?:\uDDE8|\uDDEC|\uDDF7|\uDDEA|\uDDF9|\uDDF8|\uDDED|\uDDE6|\uDDFA)|\uDDF8\uD83C(?:\uDDFB|\uDDED|\uDDF2|\uDDF9|\uDDE6|\uDDF3|\uDDE8|\uDDF1|\uDDEC|\uDDF0|\uDDEE|\uDDE7|\uDDF4|\uDDE9|\uDDF7|\uDDFF|\uDDEA|\uDDFE|\uDDEF|\uDDFD|\uDDF8)|\uDDEC\uD83C(?:\uDDF6|\uDDE6|\uDDF2|\uDDEA|\uDDED|\uDDEE|\uDDF7|\uDDF1|\uDDE9|\uDDFA|\uDDF9|\uDDF3|\uDDFC|\uDDFE|\uDDE7|\uDDEC|\uDDF8|\uDDEB|\uDDF5)|\uDDEB\uD83C(?:\uDDF0|\uDDF4|\uDDEF|\uDDEE|\uDDF7|\uDDF2)|\uDDF5\uD83C(?:\uDDEB|\uDDF0|\uDDFC|\uDDF8|\uDDE6|\uDDEC|\uDDFE|\uDDEA|\uDDED|\uDDF1|\uDDF9|\uDDF7|\uDDF3|\uDDF2)|\uDDEE\uD83C(?:\uDDF8|\uDDF3|\uDDE9|\uDDF7|\uDDF6|\uDDEA|\uDDF1|\uDDF9|\uDDF4|\uDDF2|\uDDE8)|\uDDEF\uD83C(?:\uDDF2|\uDDF5|\uDDEA|\uDDF4)|\uDDFD\uD83C\uDDF0|\uDDF1\uD83C(?:\uDDE6|\uDDFB|\uDDE7|\uDDF8|\uDDF7|\uDDFE|\uDDEE|\uDDF9|\uDDFA|\uDDE8|\uDDF0)|\uDDF2\uD83C(?:\uDDF4|\uDDF0|\uDDEC|\uDDFC|\uDDFE|\uDDFB|\uDDF1|\uDDF9|\uDDED|\uDDF7|\uDDFA|\uDDFD|\uDDE9|\uDDE8|\uDDF3|\uDDEA|\uDDF8|\uDDE6|\uDDFF|\uDDF2|\uDDF6|\uDDF5|\uDDEB)|\uDDF3\uD83C(?:\uDDE6|\uDDF7|\uDDF5|\uDDF1|\uDDE8|\uDDFF|\uDDEE|\uDDEA|\uDDEC|\uDDFA|\uDDF4|\uDDEB)|\uDDF4\uD83C\uDDF2|\uDDF6\uD83C\uDDE6|\uDDF7\uD83C(?:\uDDF4|\uDDFA|\uDDFC|\uDDF8|\uDDEA)|\uDDFB\uD83C(?:\uDDE8|\uDDEE|\uDDFA|\uDDE6|\uDDEA|\uDDF3|\uDDEC)|\uDDFC\uD83C(?:\uDDF8|\uDDEB)|\uDDFF\uD83C(?:\uDDE6|\uDDF2|\uDDFC)|\uDDFA\uD83C(?:\uDDEC|\uDDE6|\uDDF8|\uDDFE|\uDDFF|\uDDF2)|\uDDFE\uD83C(?:\uDDEA|\uDDF9)|\uDF85\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFC3\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFCA\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFC4\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFCB\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFC7\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDFF3\uD83C\uDF08)|\u270A\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\u270C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\u270B\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\u261D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uD83E(?:\uDD18\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD34\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD36\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD35\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD37\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD26\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD30\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD33\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD1E\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD19\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD1B\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD1C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD1A\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD1D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD38\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD3C\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD3D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD3E\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\uDD39\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF))|\u270D\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF)|\u26F9\uD83C(?:\uDFFB|\uDFFC|\uDFFD|\uDFFE|\uDFFF))|[\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u261D\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2648-\u2653\u2660\u2663\u2665\u2666\u2668\u267B\u267F\u2692-\u2694\u2696\u2697\u2699\u269B\u269C\u26A0\u26A1\u26AA\u26AB\u26B0\u26B1\u26BD\u26BE\u26C4\u26C5\u26C8\u26CE\u26CF\u26D1\u26D3\u26D4\u26E9\u26EA\u26F0-\u26F5\u26F7-\u26FA\u26FD\u2702\u2705\u2708-\u270D\u270F\u2712\u2714\u2716\u271D\u2721\u2728\u2733\u2734\u2744\u2747\u274C\u274E\u2753-\u2755\u2757\u2763\u2764\u2795-\u2797\u27A1\u27B0\u27BF\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55\u3030\u303D\u3297\u3299]|\uD83C[\uDC04\uDCCF\uDD70\uDD71\uDD7E\uDD7F\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE02\uDE1A\uDE2F\uDE32-\uDE3A\uDE50\uDE51\uDF00-\uDF21\uDF24-\uDF93\uDF96\uDF97\uDF99-\uDF9B\uDF9E-\uDFF0\uDFF3-\uDFF5\uDFF7-\uDFFF]|\uD83D[\uDC00-\uDCFD\uDCFF-\uDD3D\uDD49-\uDD4E\uDD50-\uDD67\uDD6F\uDD70\uDD73-\uDD7A\uDD87\uDD8A-\uDD8D\uDD90\uDD95\uDD96\uDDA4\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDE8\uDDEF\uDDF3\uDDFA-\uDE4F\uDE80-\uDEC5\uDECB-\uDED2\uDEE0-\uDEE5\uDEE9\uDEEB\uDEEC\uDEF0\uDEF3-\uDEF6]|\uD83E[\uDD10-\uDD1E\uDD20-\uDD27\uDD30\uDD33-\uDD3A\uDD3C-\uDD3E\uDD40-\uDD45\uDD47-\uDD4B\uDD50-\uDD5E\uDD80-\uDD91\uDDC0]';

exports.default = emoji;